@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2015-2025, NVIDIA CORPORATION. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,8 +25,7 @@
 #if defined(NV_DRM_AVAILABLE)
 
 #include "nvidia-drm-priv.h"
-#include "nvidia-drm-ioctl.h"
-#include "nvidia-drm-prime-fence.h"
+#include "nvidia-drm-fence.h"
 #include "nvidia-drm-gem.h"
 #include "nvidia-drm-gem-nvkms-memory.h"
 #include "nvidia-drm-gem-user-memory.h"
@@ -34,18 +33,12 @@
 #include "nvidia-drm-helper.h"
 #include "nvidia-drm-gem-dma-buf.h"
 #include "nvidia-drm-gem-nvkms-memory.h"
+#include "nv_drm_common_ioctl.h"
 
-#if defined(NV_DRM_DRM_DRV_H_PRESENT)
 #include <drm/drm_drv.h>
-#endif
-
-#if defined(NV_DRM_DRM_PRIME_H_PRESENT)
 #include <drm/drm_prime.h>
-#endif
-
-#if defined(NV_DRM_DRM_FILE_H_PRESENT)
 #include <drm/drm_file.h>
-#endif
+#include <drm/drm_vma_manager.h>
 
 #include "linux/dma-buf.h"
 
@@ -58,7 +51,7 @@ void nv_drm_gem_free(struct drm_gem_object *gem)
     /* Cleanup core gem object */
     drm_gem_object_release(&nv_gem->base);
 
-#if defined(NV_DRM_FENCE_AVAILABLE) && !defined(NV_DRM_GEM_OBJECT_HAS_RESV)
+#if !defined(NV_DRM_GEM_OBJECT_HAS_RESV)
     nv_dma_resv_fini(&nv_gem->resv);
 #endif
 
@@ -67,19 +60,33 @@ void nv_drm_gem_free(struct drm_gem_object *gem)
 
 #if !defined(NV_DRM_DRIVER_HAS_GEM_PRIME_CALLBACKS) && \
     defined(NV_DRM_GEM_OBJECT_VMAP_HAS_MAP_ARG)
+
+/*
+ * The 'dma_buf_map' structure is renamed to 'iosys_map' by the commit
+ * 7938f4218168 ("dma-buf-map: Rename to iosys-map").
+ */
+#if defined(NV_LINUX_IOSYS_MAP_H_PRESENT)
+typedef struct iosys_map nv_sysio_map_t;
+#else
+typedef struct dma_buf_map nv_sysio_map_t;
+#endif
+
 static int nv_drm_gem_vmap(struct drm_gem_object *gem,
-                           struct dma_buf_map *map)
+                           nv_sysio_map_t *map)
 {
-    map->vaddr = nv_drm_gem_prime_vmap(gem);
-    if (map->vaddr == NULL) {
+    void *vaddr = nv_drm_gem_prime_vmap(gem);
+    if (vaddr == NULL) {
         return -ENOMEM;
+    } else if (IS_ERR(vaddr)) {
+        return PTR_ERR(vaddr);
     }
+    map->vaddr = vaddr;
     map->is_iomem = true;
     return 0;
 }
 
 static void nv_drm_gem_vunmap(struct drm_gem_object *gem,
-                              struct dma_buf_map *map)
+                              nv_sysio_map_t *map)
 {
     nv_drm_gem_prime_vunmap(gem, map->vaddr);
     map->vaddr = NULL;
@@ -121,13 +128,8 @@ void nv_drm_gem_object_init(struct nv_drm_device *nv_dev,
 
     /* Initialize the gem object */
 
-#if defined(NV_DRM_FENCE_AVAILABLE)
+#if !defined(NV_DRM_GEM_OBJECT_HAS_RESV)
     nv_dma_resv_init(&nv_gem->resv);
-
-#if defined(NV_DRM_GEM_OBJECT_HAS_RESV)
-    nv_gem->base.resv = &nv_gem->resv;
-#endif
-
 #endif
 
 #if !defined(NV_DRM_DRIVER_HAS_GEM_FREE_OBJECT)
@@ -135,12 +137,17 @@ void nv_drm_gem_object_init(struct nv_drm_device *nv_dev,
 #endif
 
     drm_gem_private_object_init(dev, &nv_gem->base, size);
+
+    /* Create mmap offset early for drm_gem_prime_mmap(), if possible. */
+    if (nv_gem->ops->create_mmap_offset) {
+        uint64_t offset;
+        nv_gem->ops->create_mmap_offset(nv_dev, nv_gem, &offset);
+    }
 }
 
 struct drm_gem_object *nv_drm_gem_prime_import(struct drm_device *dev,
                                                struct dma_buf *dma_buf)
 {
-#if defined(NV_DMA_BUF_OWNER_PRESENT)
     struct drm_gem_object *gem_dst;
     struct nv_drm_gem_object *nv_gem_src;
 
@@ -157,11 +164,13 @@ struct drm_gem_object *nv_drm_gem_prime_import(struct drm_device *dev,
              */
             gem_dst = nv_gem_src->ops->prime_dup(dev, nv_gem_src);
 
-            if (gem_dst)
-                return gem_dst;
+            if (gem_dst == NULL) {
+                return ERR_PTR(-ENOTSUPP);
+            }
+
+            return gem_dst;
         }
     }
-#endif /* NV_DMA_BUF_OWNER_PRESENT */
 
     return drm_gem_prime_import(dev, dma_buf);
 }
@@ -201,8 +210,7 @@ void nv_drm_gem_prime_vunmap(struct drm_gem_object *gem, void *address)
 nv_dma_resv_t* nv_drm_gem_prime_res_obj(struct drm_gem_object *obj)
 {
     struct nv_drm_gem_object *nv_gem = to_nv_gem_object(obj);
-
-    return &nv_gem->resv;
+    return nv_drm_gem_res_obj(nv_gem);
 }
 #endif
 
@@ -214,8 +222,7 @@ int nv_drm_gem_map_offset_ioctl(struct drm_device *dev,
     struct nv_drm_gem_object *nv_gem;
     int ret;
 
-    if ((nv_gem = nv_drm_gem_object_lookup(dev,
-                                           filep,
+    if ((nv_gem = nv_drm_gem_object_lookup(filep,
                                            params->handle)) == NULL) {
         NV_DRM_DEV_LOG_ERR(
             nv_dev,
@@ -224,6 +231,7 @@ int nv_drm_gem_map_offset_ioctl(struct drm_device *dev,
         return -EINVAL;
     }
 
+    /* mmap offset creation is idempotent, fetch it by creating it again. */
     if (nv_gem->ops->create_mmap_offset) {
         ret = nv_gem->ops->create_mmap_offset(nv_dev, nv_gem, &params->offset);
     } else {
@@ -239,7 +247,6 @@ int nv_drm_gem_map_offset_ioctl(struct drm_device *dev,
     return ret;
 }
 
-#if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
 int nv_drm_mmap(struct file *file, struct vm_area_struct *vma)
 {
     struct drm_file *priv = file->private_data;
@@ -250,8 +257,8 @@ int nv_drm_mmap(struct file *file, struct vm_area_struct *vma)
     struct nv_drm_gem_object *nv_gem;
 
     drm_vma_offset_lock_lookup(dev->vma_offset_manager);
-    node = nv_drm_vma_offset_exact_lookup_locked(dev->vma_offset_manager,
-                                                 vma->vm_pgoff, vma_pages(vma));
+    node = drm_vma_offset_exact_lookup_locked(dev->vma_offset_manager,
+                                              vma->vm_pgoff, vma_pages(vma));
     if (likely(node)) {
         obj = container_of(node, struct drm_gem_object, vma_node);
         /*
@@ -277,7 +284,7 @@ int nv_drm_mmap(struct file *file, struct vm_area_struct *vma)
         goto done;
     }
 
-    if (!nv_drm_vma_node_is_allowed(node, file)) {
+    if (!drm_vma_node_is_allowed(node, file->private_data)) {
         ret = -EACCES;
         goto done;
     }
@@ -288,7 +295,7 @@ int nv_drm_mmap(struct file *file, struct vm_area_struct *vma)
             ret = -EINVAL;
             goto done;
         }
-        vma->vm_flags &= ~VM_MAYWRITE;
+        nv_vm_flags_clear(vma, VM_MAYWRITE);
     }
 #endif
 
@@ -299,29 +306,47 @@ done:
 
     return ret;
 }
-#endif
 
 int nv_drm_gem_identify_object_ioctl(struct drm_device *dev,
                                      void *data, struct drm_file *filep)
 {
     struct drm_nvidia_gem_identify_object_params *p = data;
+    struct nv_drm_gem_dma_buf *nv_dma_buf;
+    struct nv_drm_gem_nvkms_memory *nv_nvkms_memory;
+    struct nv_drm_gem_user_memory *nv_user_memory;
+    struct nv_drm_gem_object *nv_gem = NULL;
 
     if (!drm_core_check_feature(dev, DRIVER_MODESET)) {
-        return -EINVAL;
+        return -EOPNOTSUPP;
     }
 
-    if (nv_drm_gem_object_dma_buf_lookup(dev, filep, p->handle) != NULL) {
+    nv_dma_buf = nv_drm_gem_object_dma_buf_lookup(filep, p->handle);
+    if (nv_dma_buf) {
         p->object_type = NV_GEM_OBJECT_DMABUF;
-#if defined(NV_DRM_ATOMIC_MODESET_AVAILABLE)
-    } else if (nv_drm_gem_object_nvkms_memory_lookup(dev, filep, p->handle) != NULL) {
-        p->object_type = NV_GEM_OBJECT_NVKMS;
-#endif
-    } else if (nv_drm_gem_object_user_memory_lookup(dev, filep, p->handle) != NULL) {
-        p->object_type = NV_GEM_OBJECT_USERMEMORY;
-    } else {
-        p->object_type = NV_GEM_OBJECT_UNKNOWN;
+        nv_gem = &nv_dma_buf->base;
+        goto done;
     }
 
+    nv_nvkms_memory = nv_drm_gem_object_nvkms_memory_lookup(filep, p->handle);
+    if (nv_nvkms_memory) {
+        p->object_type = NV_GEM_OBJECT_NVKMS;
+        nv_gem = &nv_nvkms_memory->base;
+        goto done;
+    }
+
+    nv_user_memory = nv_drm_gem_object_user_memory_lookup(filep, p->handle);
+    if (nv_user_memory) {
+        p->object_type = NV_GEM_OBJECT_USERMEMORY;
+        nv_gem = &nv_user_memory->base;
+        goto done;
+    }
+
+    p->object_type = NV_GEM_OBJECT_UNKNOWN;
+
+done:
+    if (nv_gem) {
+        nv_drm_gem_object_unreference_unlocked(nv_gem);
+    }
     return 0;
 }
 
